# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'rare/ui/components/tabs/settings/widgets/env_vars.ui'
#
# Created by: PyQt5 UI code generator 5.15.6
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_EnvVars(object):
    def setupUi(self, EnvVars):
        EnvVars.setObjectName("EnvVars")
        EnvVars.resize(720, 561)
        EnvVars.setMinimumSize(QtCore.QSize(0, 200))
        EnvVars.setWindowTitle("GroupBox")
        self.verticalLayout = QtWidgets.QVBoxLayout(EnvVars)
        self.verticalLayout.setObjectName("verticalLayout")
        self.env_vars_table = QtWidgets.QTableWidget(EnvVars)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.env_vars_table.sizePolicy().hasHeightForWidth())
        self.env_vars_table.setSizePolicy(sizePolicy)
        self.env_vars_table.setVerticalScrollBarPolicy(QtCore.Qt.ScrollBarAsNeeded)
        self.env_vars_table.setSizeAdjustPolicy(QtWidgets.QAbstractScrollArea.AdjustToContents)
        self.env_vars_table.setObjectName("env_vars_table")
        self.env_vars_table.setColumnCount(2)
        self.env_vars_table.setRowCount(1)
        item = QtWidgets.QTableWidgetItem()
        item.setText("-")
        self.env_vars_table.setVerticalHeaderItem(0, item)
        item = QtWidgets.QTableWidgetItem()
        self.env_vars_table.setHorizontalHeaderItem(0, item)
        item = QtWidgets.QTableWidgetItem()
        self.env_vars_table.setHorizontalHeaderItem(1, item)
        self.env_vars_table.horizontalHeader().setStretchLastSection(True)
        self.env_vars_table.verticalHeader().setStretchLastSection(False)
        self.verticalLayout.addWidget(self.env_vars_table)

        self.retranslateUi(EnvVars)
        QtCore.QMetaObject.connectSlotsByName(EnvVars)

    def retranslateUi(self, EnvVars):
        _translate = QtCore.QCoreApplication.translate
        EnvVars.setTitle(_translate("EnvVars", "Environment variables"))
        item = self.env_vars_table.horizontalHeaderItem(0)
        item.setText(_translate("EnvVars", "Key"))
        item = self.env_vars_table.horizontalHeaderItem(1)
        item.setText(_translate("EnvVars", "Value"))


if __name__ == "__main__":
    import sys
    app = QtWidgets.QApplication(sys.argv)
    EnvVars = QtWidgets.QGroupBox()
    ui = Ui_EnvVars()
    ui.setupUi(EnvVars)
    EnvVars.show()
    sys.exit(app.exec_())
