# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'rare/ui/components/tabs/downloads/downloads_tab.ui'
#
# Created by: PyQt5 UI code generator 5.15.6
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_DownloadsTab(object):
    def setupUi(self, DownloadsTab):
        DownloadsTab.setObjectName("DownloadsTab")
        DownloadsTab.resize(677, 195)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(DownloadsTab.sizePolicy().hasHeightForWidth())
        DownloadsTab.setSizePolicy(sizePolicy)
        DownloadsTab.setWindowTitle("Form")
        self.downloads_tab_layout = QtWidgets.QVBoxLayout(DownloadsTab)
        self.downloads_tab_layout.setObjectName("downloads_tab_layout")
        self.download_layout = QtWidgets.QGridLayout()
        self.download_layout.setObjectName("download_layout")
        self.dl_speed_label = QtWidgets.QLabel(DownloadsTab)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.dl_speed_label.setFont(font)
        self.dl_speed_label.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.dl_speed_label.setObjectName("dl_speed_label")
        self.download_layout.addWidget(self.dl_speed_label, 1, 2, 1, 1)
        self.dl_speed = QtWidgets.QLabel(DownloadsTab)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.dl_speed.sizePolicy().hasHeightForWidth())
        self.dl_speed.setSizePolicy(sizePolicy)
        self.dl_speed.setText("dl_speed")
        self.dl_speed.setObjectName("dl_speed")
        self.download_layout.addWidget(self.dl_speed, 1, 3, 1, 1)
        self.cache_used = QtWidgets.QLabel(DownloadsTab)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Maximum)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.cache_used.sizePolicy().hasHeightForWidth())
        self.cache_used.setSizePolicy(sizePolicy)
        self.cache_used.setText("cache_used")
        self.cache_used.setObjectName("cache_used")
        self.download_layout.addWidget(self.cache_used, 2, 1, 1, 1)
        self.time_left = QtWidgets.QLabel(DownloadsTab)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Maximum)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.time_left.sizePolicy().hasHeightForWidth())
        self.time_left.setSizePolicy(sizePolicy)
        self.time_left.setText("time_left")
        self.time_left.setObjectName("time_left")
        self.download_layout.addWidget(self.time_left, 1, 1, 1, 1)
        self.dl_name = QtWidgets.QLabel(DownloadsTab)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Maximum)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.dl_name.sizePolicy().hasHeightForWidth())
        self.dl_name.setSizePolicy(sizePolicy)
        font = QtGui.QFont()
        font.setBold(False)
        font.setWeight(50)
        self.dl_name.setFont(font)
        self.dl_name.setText("dl_name")
        self.dl_name.setObjectName("dl_name")
        self.download_layout.addWidget(self.dl_name, 0, 1, 1, 3)
        self.dl_name_label = QtWidgets.QLabel(DownloadsTab)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.dl_name_label.setFont(font)
        self.dl_name_label.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.dl_name_label.setObjectName("dl_name_label")
        self.download_layout.addWidget(self.dl_name_label, 0, 0, 1, 1)
        self.time_left_label = QtWidgets.QLabel(DownloadsTab)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.time_left_label.setFont(font)
        self.time_left_label.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.time_left_label.setObjectName("time_left_label")
        self.download_layout.addWidget(self.time_left_label, 1, 0, 1, 1)
        self.downloaded_label = QtWidgets.QLabel(DownloadsTab)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.downloaded_label.setFont(font)
        self.downloaded_label.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.downloaded_label.setObjectName("downloaded_label")
        self.download_layout.addWidget(self.downloaded_label, 2, 2, 1, 1)
        self.downloaded = QtWidgets.QLabel(DownloadsTab)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.downloaded.sizePolicy().hasHeightForWidth())
        self.downloaded.setSizePolicy(sizePolicy)
        self.downloaded.setText("downloaded")
        self.downloaded.setObjectName("downloaded")
        self.download_layout.addWidget(self.downloaded, 2, 3, 1, 1)
        self.cache_used_label = QtWidgets.QLabel(DownloadsTab)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.cache_used_label.setFont(font)
        self.cache_used_label.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.cache_used_label.setObjectName("cache_used_label")
        self.download_layout.addWidget(self.cache_used_label, 2, 0, 1, 1)
        self.download_layout.setColumnStretch(1, 1)
        self.download_layout.setColumnStretch(3, 1)
        self.downloads_tab_layout.addLayout(self.download_layout)
        self.progress_layout = QtWidgets.QHBoxLayout()
        self.progress_layout.setObjectName("progress_layout")
        self.progress_bar = QtWidgets.QProgressBar(DownloadsTab)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Maximum)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.progress_bar.sizePolicy().hasHeightForWidth())
        self.progress_bar.setSizePolicy(sizePolicy)
        self.progress_bar.setProperty("value", 0)
        self.progress_bar.setObjectName("progress_bar")
        self.progress_layout.addWidget(self.progress_bar)
        self.kill_button = QtWidgets.QPushButton(DownloadsTab)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Maximum, QtWidgets.QSizePolicy.Maximum)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.kill_button.sizePolicy().hasHeightForWidth())
        self.kill_button.setSizePolicy(sizePolicy)
        self.kill_button.setObjectName("kill_button")
        self.progress_layout.addWidget(self.kill_button)
        self.downloads_tab_layout.addLayout(self.progress_layout)
        self.queue_scroll = QtWidgets.QScrollArea(DownloadsTab)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.queue_scroll.sizePolicy().hasHeightForWidth())
        self.queue_scroll.setSizePolicy(sizePolicy)
        self.queue_scroll.setFocusPolicy(QtCore.Qt.WheelFocus)
        self.queue_scroll.setFrameShape(QtWidgets.QFrame.NoFrame)
        self.queue_scroll.setHorizontalScrollBarPolicy(QtCore.Qt.ScrollBarAlwaysOff)
        self.queue_scroll.setWidgetResizable(True)
        self.queue_scroll.setAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignTop)
        self.queue_scroll.setObjectName("queue_scroll")
        self.queue_scroll_contents = QtWidgets.QWidget()
        self.queue_scroll_contents.setGeometry(QtCore.QRect(0, 0, 665, 16))
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Maximum)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.queue_scroll_contents.sizePolicy().hasHeightForWidth())
        self.queue_scroll_contents.setSizePolicy(sizePolicy)
        self.queue_scroll_contents.setObjectName("queue_scroll_contents")
        self.queue_scroll_contents_layout = QtWidgets.QVBoxLayout(self.queue_scroll_contents)
        self.queue_scroll_contents_layout.setContentsMargins(0, 0, 9, 0)
        self.queue_scroll_contents_layout.setObjectName("queue_scroll_contents_layout")
        self.queue_scroll.setWidget(self.queue_scroll_contents)
        self.downloads_tab_layout.addWidget(self.queue_scroll)

        self.retranslateUi(DownloadsTab)
        QtCore.QMetaObject.connectSlotsByName(DownloadsTab)

    def retranslateUi(self, DownloadsTab):
        _translate = QtCore.QCoreApplication.translate
        self.dl_speed_label.setText(_translate("DownloadsTab", "Download speed:"))
        self.dl_name_label.setText(_translate("DownloadsTab", "Current download:"))
        self.time_left_label.setText(_translate("DownloadsTab", "Time left:"))
        self.downloaded_label.setText(_translate("DownloadsTab", "Downloaded:"))
        self.cache_used_label.setText(_translate("DownloadsTab", "Cache size:"))
        self.kill_button.setText(_translate("DownloadsTab", "Stop Download"))


if __name__ == "__main__":
    import sys
    app = QtWidgets.QApplication(sys.argv)
    DownloadsTab = QtWidgets.QWidget()
    ui = Ui_DownloadsTab()
    ui.setupUi(DownloadsTab)
    DownloadsTab.show()
    sys.exit(app.exec_())
